/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.tuple.api.Tuple;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
// Import clauses required by the object Olstore_type
import jorm.Olstore_typeAccessor;
// Imports needed by the rdb mapper
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.io.IOException;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;

public class Olstore_typeBinding
	extends org.objectweb.jonas_ejb.container.JEntitySwitchCS
	implements PBindingCtrl
      {
	/**
	 * The current state of this PBinding object.
	 */
	transient protected byte status;
	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	transient public Olstore_typeMapping pClassMapping;
	/**
	 * The PName of this PBinding object.
	 */
	transient protected PName pname;
	transient RdbAdapter adapter;

	public Olstore_typeBinding() {
// nothing
	}

	public Olstore_typeBinding(PClassMapping pcm) {
		pClassMapping = (Olstore_typeMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (Olstore_typeMapping) pcm;
		status = PBinding.LIFECYCLE_NOTBOUND;
	    pname = null;
		if (pcm != null) {
			PMapperRdb mapper = (PMapperRdb) pcm.getPMapper();
			if (mapper != null) {
				adapter = mapper.getRdbAdapter();
			}
		}
		if (adapter == null) {
			throw new PException("No mapper found throught the PClassMapping: " + pcm);
		}
	}

	public void write(Object conn, PAccessor pa)
		throws PException {
		Olstore_typeAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Olstore_typeAccessor)
			paccessor = (Olstore_typeAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		    if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
//Table olstore_type : readOnly=false mustGenerateUpdate=false colocatedTable=false joins=$tab.joins
                sqlquery = deleteolstore_typeTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_type]: problematic SQL query -> {" + sqlquery + "}");
                }
            } else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertolstore_typeTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_type]: problematic SQL query -> {" + sqlquery + "}");
                }
		    } else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
		    }
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Read");
		Olstore_typeAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Olstore_typeAccessor)
			paccessor = (Olstore_typeAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            pstmt = connection.prepareStatement(pClassMapping.readQuery);
            adapter.setString(pstmt, 1, pname.encodeString());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetName(adapter.getString(resultSet, 1, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@4845aa
				//fieldName=name
                fieldval = adapter.getString(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.jonasCMRItem_TypePnc.decodeString(null);
                } else {
                    refname = pClassMapping.jonasCMRItem_TypePnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetJonasCMRItem_Type(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@14eaec9
				//fieldName=name
                fieldval = adapter.getString(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.propertiesPnc.decodeString(null);
                } else {
                    refname = pClassMapping.propertiesPnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetProperties(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetStringField(pClassMapping.name_NAME, adapter.getString(resultSet, 1, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@14eaec9
				//fieldName=name
                fieldval = adapter.getString(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.propertiesPnc.decodeString(null);
                } else {
                    refname = pClassMapping.propertiesPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.properties_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@14eaec9
				//fieldName=name
                fieldval = adapter.getString(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.propertiesPnc.decodeString(null);
                } else {
                    refname = pClassMapping.propertiesPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.properties_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		status = nextState;
	}

	public void read(Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = pClassMapping.getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(conn, pa);
	        return;
	    }
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, pClassMapping, txctx);
	        if (tuple == null) {
	            read(conn, pa);
	            return;
	        }
		    Olstore_typeAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof Olstore_typeAccessor)
			    paccessor = (Olstore_typeAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
				fval =
					tuple.getString(1);
                if (tuple.isDefined(1)) {
                    paccessor.paSetJonasCMRItem_Type(pClassMapping.jonasCMRItem_TypePnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetJonasCMRItem_Type(pClassMapping.jonasCMRItem_TypePnc.getNull(), conn);
                }
                paccessor.paSetName(tuple.getString(1));
				fval =
					tuple.getString(1);
                if (tuple.isDefined(1)) {
                    paccessor.paSetProperties(pClassMapping.propertiesPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetProperties(pClassMapping.propertiesPnc.getNull(), conn);
                }
            } else if (paccessorGen != null) {
				fval =
					tuple.getString(1);
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(pClassMapping.jonasCMRItem_Type_NAME, pClassMapping.jonasCMRItem_TypePnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.jonasCMRItem_Type_NAME, pClassMapping.jonasCMRItem_TypePnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(pClassMapping.name_NAME, tuple.getString(1));
				fval =
					tuple.getString(1);
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(pClassMapping.properties_NAME, pClassMapping.propertiesPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.properties_NAME, pClassMapping.propertiesPnc.getNull(), conn);
                }
            }
		    status = nextState;
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Exist");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		status = nextState;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT olstore_type.name FROM olstore_type WHERE olstore_type.name = ?");
            adapter.setString(pstmt, 1, pname.encodeString());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}

	public PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export with hints");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unexport");
        pClassMapping.getPBinder().unexport(conn, this.pname, this);
	}

	public void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Bind");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unbind");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}
    private String insertolstore_typeTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                Olstore_typeAccessor paccessor)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO olstore_type (name) VALUES (?)");
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO olstore_type (name) VALUES (?)");
				//name
                nullField = paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteolstore_typeTable(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	Olstore_typeAccessor paccessor) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM olstore_type WHERE name = ?");
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
}

















